==> muffin-0.0.81-py2.py3-none-any.whl: muffin/utils.py <==
import hmac
import hashlib
import random
import asyncio


SALT_CHARS = 'bcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'


class MuffinException(Exception):

    """ Implement a Muffin's exception. """

    pass


def abcoroutine(func):
    """ Mark function/method as coroutine.

    Used with Meta classes.

    """
    func._abcoroutine = True
    return func


def to_coroutine(func):
    """ Ensure that the function is coroutine.

    If not convert to coroutine.

    """
    if not asyncio.iscoroutinefunction(func):
        func = asyncio.coroutine(func)
    return func


def create_signature(secret, value, digestmod='sha1', encoding='utf-8'):
    """ Create HMAC Signature from secret for value. """
    if isinstance(secret, str):
        secret = secret.encode(encoding)

    if isinstance(value, str):
        value = value.encode(encoding)

    if isinstance(digestmod, str):
        digestmod = getattr(hashlib, digestmod, hashlib.sha1)

    hm = hmac.new(secret, digestmod=hashlib.sha1)
    hm.update(value)
    return hm.hexdigest()


def check_signature(signature, *args, **kwargs):
    """ Check for the signature is correct. """
    return hmac.compare_digest(signature, create_signature(*args, **kwargs))


def generate_password_hash(password, digestmod='sha1', salt_length=8):
    """ Hash a password with given method and salt length. """

    salt = ''.join(random.sample(SALT_CHARS, salt_length))
    signature = create_signature(salt, password, digestmod=digestmod)
    return '$'.join((digestmod, salt, signature))


def check_password_hash(password, pwhash):
    if pwhash.count('$') < 2:
        return False
    digestmod, salt, signature = pwhash.split('$', 2)
    return check_signature(signature, salt, password, digestmod=digestmod)


class Structure(dict):

    """ `Attribute` dictionary. """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("Attribute '%s' doesn't exists. " % name)

    def __setattr__(self, name, value):
        self[name] = value

==> muffin-0.0.98-py2.py3-none-any.whl: muffin/utils.py <==
import asyncio
import hashlib
import hmac
import random


SALT_CHARS = 'bcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'


class MuffinException(Exception):

    """ Implement a Muffin's exception. """

    pass


def abcoroutine(func):
    """ Mark function/method as coroutine.

    Used with Meta classes.

    """
    func._abcoroutine = True
    return func


def to_coroutine(func):
    """ Ensure that the function is coroutine.

    If not convert to coroutine.

    """
    if not asyncio.iscoroutinefunction(func):
        func = asyncio.coroutine(func)
    return func


def create_signature(secret, value, digestmod='sha1', encoding='utf-8'):
    """ Create HMAC Signature from secret for value. """
    if isinstance(secret, str):
        secret = secret.encode(encoding)

    if isinstance(value, str):
        value = value.encode(encoding)

    if isinstance(digestmod, str):
        digestmod = getattr(hashlib, digestmod, hashlib.sha1)

    hm = hmac.new(secret, digestmod=hashlib.sha1)
    hm.update(value)
    return hm.hexdigest()


def check_signature(signature, *args, **kwargs):
    """ Check for the signature is correct. """
    return hmac.compare_digest(signature, create_signature(*args, **kwargs))


def generate_password_hash(password, digestmod='sha1', salt_length=8):
    """ Hash a password with given method and salt length. """

    salt = ''.join(random.sample(SALT_CHARS, salt_length))
    signature = create_signature(salt, password, digestmod=digestmod)
    return '$'.join((digestmod, salt, signature))


def check_password_hash(password, pwhash):
    if pwhash.count('$') < 2:
        return False
    digestmod, salt, signature = pwhash.split('$', 2)
    return check_signature(signature, salt, password, digestmod=digestmod)


class Structure(dict):

    """ `Attribute` dictionary. Use attributes as keys. """

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError:
            raise AttributeError("Attribute '%s' doesn't exists. " % name)

    def __setattr__(self, name, value):
        self[name] = value


class local:

    """ coroutine.local storage is simular to python's threading.local.

    Usage: ::

        local = muffin.local(loop)
        local.value = 42

    """

    __slots__ = '_loop',
    __locals__ = {}

    def __new__(cls, loop=None):
        key = id(loop)
        if key not in cls.__locals__:
            cls.__locals__[key] = object.__new__(cls)
        return cls.__locals__[key]

    def __init__(self, loop=None):
        object.__setattr__(self, '_loop', loop or asyncio.get_event_loop())

    def __getattr__(self, name):
        try:
            return self.__local__[name]
        except KeyError:
            raise AttributeError

    def __setattr__(self, name, value):
        self.__local__[name] = value

    @property
    def __local__(self):
        """ Create namespace in current task. """
        task = asyncio.Task.current_task(loop=self._loop)
        if not task:
            raise RuntimeError('No one running tasks were found.')

        if not hasattr(task, '_local'):
            task._local = {}
        return task._local
